#include "ti86asm.inc"


.org _asm_exec_ram
    nop
    jp start
    .dw 0
    .dw Title
Title:
    .db     "Fastrun 3.1 "$10,"KEY installer",0 ;10 is sqrt sign

start:
;check [sqrt]KEY state
    ld hl,_Flags+$24
    bit 6,(hl)                      ;is [sqrt]KEY enabled?
    res 6,(hl)                      ;however, disable it
    jr nz,disabled                  ;if it was enabled, then say it has been disabled
    
;else install and enable it...
;find program
    ld hl,$5a92                     ;name of [sqrt]KEY program
    rst 20h                         ;[sqrt]KEY -> OP1
    rst 10h                         ;_findsym
    call nc,_delvar                 ;delete if program allready exists

;prepare copy
    ld hl, code_end - code_start    ;hl = size of program
    push hl                         ;save the size
;create program
    call _CREATEPROG                ;create program
;set destionation
    ld a,b
    ex de,hl                        ;get address to program to ahl
    call _AHL_PLUS_2_PG3            ;skip two lenght bytes
    call _SET_ABS_DEST_ADDR         ;set as destination for copy
;set lenght    
    xor a
    pop hl                          ;code length
    call _SET_MM_NUM_BYTES          ;set the nubmer of bytes to copy, doesn't affect a
;set source    

    ld hl,code
    call _SET_ABS_SRC_ADDR          ;set the source
;copy
    call _mm_ldir                   ;copy it
;enable [sqrt]KEY
    set 6,(iy+$24)                  ;enable [sqrt]KEY program
    ld hl,enabled_t                 ;"Enabled"
;    jr disp_it                      ;print the message
    jp _puts
    
disabled:
    ld hl,disabled_t                ;"Disabled"
disp_it:
    call _puts                      ;print sting at hl
    jp _newline                     ;go to next line and return
    
enabled_t:  ;---------------------; Screen width
    .db     "Fastrun enabled,     "
    .db     "press 2nd + Exit to  "
    .db     "run command in fstrun",0
disabled_t:
    .text "Disabled"


;the [sqrt]KEY program
code:
.org _asm_exec_ram-2
code_start:
.db $8e,$28                         ;assembly program token

    call _POPOP1                    ;pop op1, needed in all [sqrt]KEY programs
    ld a,(_asm_reg_a)     
    or a                            ;aka cp 0
    ret z                           ;leave if no keypress (speeds up OS)
    
    push af                         ;save the keypress

;check exit key
    cp K_EXIT
    jr nz,end                       ;exit if not kexit presed
    
;for 2nd mode
    ld hl,_Flags+$12
    bit 3,(hl)
    jp z,end                        ;exit if not in 2nd mode
    res 3,(hl)                      ;get out of 2nd mode
    
;check for homescreen
    ld a,(_CXCURAPP)
    cp 1
    jr nz,end                       ;exit if not at homescreen
    
;running a program also counts as homescreen
    ld a,(_FPS)                     ;this byte is zero if no prog is running
    or a
    jr nz,end                       ;exit if in a program
    

;run the program

;check if fstrun exists
    ld hl,filename-1
    rst 20h                         ;name to op1
    rst 10h                         ;find program
    jr c,no_file                    ;exit if the the file "fstrun" doesn't exist
    
;copy content
    ld a,b
    ex de,hl                        ;pointer to file to ahl
    call _get_word_ahl              ;get size
    

    
;set source
    call _set_abs_src_addr          ;doesn't affect de
    

    call _load_ram_ahl              ;swap in pages and make hl point to beginning of string
    
    ld a,d
    or e
    jr z,end    
    
    ld b,e                          ;lenght as loop counter (we suppose the command is < 256 bytes)
put_command:
    ld a,(hl)
    call _putc
    inc hl
    bit 6,h                         ;bit set if hl reached $c000
    jr z,dont_change_page           ;if not, then don't swap pages
;change page when needed
    ld h,$40                        ;make hl point to beginning of the page
    in a,(6)                        ;get page number
    inc a                           ;next page
    out (6),a                       ;swap
dont_change_page:
    djnz put_command
    
;set size
    xor a
    ex de,hl                        ;size was in de
    call _set_mm_num_bytes          ;set size
    
;set destination
    ld a,(_P_editTop)               ;here's the beginning of command written at homescreen
    ld hl,(_P_editTop+1)
    call _set_abs_dest_addr         ;set this as destination
;copy
    call _mm_ldir                   ;copy the file
    
;set end of command
    call _LOAD_ABS_DEST_ADDR        ;here the copy ended, this is the end of the command copied
    call _SET_EDITCURSOR            ;copy ahl to _P_EDITCURSOR, the end of inputted command


;exit with enter pressed
    pop af                          ;restore stack
    ld a,K_ENTER                    ;return this key
    cp a                            ;set z flag
    ret                             ;exit
    
filename:
    .db 6,"fstrun"                  ;name of file to look for
    
;say that the file wasn't found
no_file:
    ld hl,no_file_t
    call _puts                      ;"Can't find fstrun"
    call _newline    
end:
;exit to OS
    pop af                          ;restore presssed key
    cp a                            ;set z flag
    ret                             ;exit

no_file_t:
    .text "Can't find fstrun"
        
code_end:

.end